require "Window"
require "Apollo"
require "GameLib"
require "ApolloColor"

local LUI_Frames = {}
local karCurrency = {
    {eType = Money.CodeEnumCurrencyType.Renown,                         strTitle = Apollo.GetString("CRB_Renown"),                       strDescription = Apollo.GetString("CRB_Renown_Desc")},
    {eType = Money.CodeEnumCurrencyType.ElderGems,                      strTitle = Apollo.GetString("CRB_Elder_Gems"),                   strDescription = Apollo.GetString("CRB_Elder_Gems_Desc")},
    {eType = Money.CodeEnumCurrencyType.Glory,                          strTitle = Apollo.GetString("CRB_Glory"),                        strDescription = Apollo.GetString("CRB_Glory_Desc")},
    {eType = Money.CodeEnumCurrencyType.Prestige,                       strTitle = Apollo.GetString("CRB_Prestige"),                     strDescription = Apollo.GetString("CRB_Prestige_Desc")},
    {eType = Money.CodeEnumCurrencyType.CraftingVouchers,               strTitle = Apollo.GetString("CRB_Crafting_Vouchers"),            strDescription = Apollo.GetString("CRB_Crafting_Voucher_Desc")},
    {eType = AccountItemLib.CodeEnumAccountCurrency.Protobucks,         strTitle = Apollo.GetString("CRB_Protobucks"),                   strDescription = Apollo.GetString("CRB_Protobucks"),                       bAccountItem = true},
    {eType = AccountItemLib.CodeEnumAccountCurrency.Omnibits,           strTitle = Apollo.GetString("CRB_OmniBits"),                     strDescription = Apollo.GetString("CRB_OmniBits_Desc"),                    bAccountItem = true},
    {eType = AccountItemLib.CodeEnumAccountCurrency.PromissoryNote,     strTitle = "Promissory Notes",                                   strDescription = "Promissory Notes",                                       bAccountItem = true},
    {eType = AccountItemLib.CodeEnumAccountCurrency.ServiceToken,       strTitle = Apollo.GetString("AccountInventory_ServiceToken"),    strDescription = Apollo.GetString("AccountInventory_ServiceToken_Desc"),   bAccountItem = true},
    {eType = AccountItemLib.CodeEnumAccountCurrency.MysticShiny,        strTitle = Apollo.GetString("CRB_FortuneCoin"),                  strDescription = Apollo.GetString("CRB_FortuneCoin_Desc"),                 bAccountItem = true},
    {eType = Money.CodeEnumCurrencyType.Triploons,                      strTitle = Apollo.GetString("CRB_Triploons"),                    strDescription = Apollo.GetString("CRB_Triploons_Desc")},
}

function LUI_Frames:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    self.settings = {
        scale = false,
        interval = 300,
        currency = 6,
        spacing = 20,
        showShadow = true,
        cubeSize = 13,
        thickness = 3,
        strata = 1,
        useUIScale = true,
        actionbar = {
            enable = true,
            showLines = true,
            elevate = false,
            width = 426,
            height = 49,
            posX = -212,
            posY = -34,
            line_length = 290,
            line_up = 25,
            line_side = 100,
            line_margin = 150,
        },
        sidebar = {
            enable = true,
            width = 39,
            height = 446,
            posX = -21,
            posY = -220,
        },
        chat = {
            showLines = true,
            showStore = true,
            showMadameFay = false,
            showEvent = true,
            strata = 3,
            posX = 43,
            posY = -38,
            length_horizontal = 570,
            length_vertical = 260,
        },
        info = {
            showLines = true,
            showXP = true,
            showTime = false,
            strata = 3,
            posX = -43,
            posY = -38,
            length_horizontal = 570,
            length_vertical = 260,
            b24HourFormat = true,
        },
        minimap = {
            enable = true,
            showLines = true,
            showConnectors = true,
            width = 200,
            height = 200,
            posX = -20,
            posY = 20,
            length_horizontal = 260,
            length_vertical = 260,
        },
        tPinnedAddons = {
            Apollo.GetString("InterfaceMenu_AccountInventory"),
            Apollo.GetString("InterfaceMenu_Character"),
            Apollo.GetString("InterfaceMenu_AbilityBuilder"),
            Apollo.GetString("InterfaceMenu_QuestLog"),
            Apollo.GetString("InterfaceMenu_GroupFinder"),
            Apollo.GetString("InterfaceMenu_Social"),
            Apollo.GetString("InterfaceMenu_Mail"),
            Apollo.GetString("InterfaceMenu_Lore"),
        },
    }
    self.strata = {
        [0] = "InWorldHudStratum",
        [1] = "FixedHudStratumLow",
        [2] = "FixedHudStratum",
        [3] = "FixedHudStratumHigh",
        [4] = "DefaultStratum",
        [5] = "TooltipStratum"
    }
    return o
end

function LUI_Frames:Init()
    Apollo.RegisterAddon(self, true, "LUI Frames", nil)
end

function LUI_Frames:OnLoad()
    Apollo.LoadSprites("sprites.xml")
    self.xmlDoc = XmlDoc.CreateFromFile("LUI_Frames.xml")
    self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

function LUI_Frames:OnDocLoaded()
    if self.xmlDoc == nil then
        return
    end

    self.shadow = Apollo.GetAddon("ActionBarFrame")
    self:LoadSettings()
    self:Build()
    self:OnCharacterCreated()

    -- General
    Apollo.RegisterEventHandler("CharacterCreated",                 "OnCharacterCreated", self)
    Apollo.RegisterEventHandler("PlayerEnteredWorld",               "OnCharacterCreated", self)
    Apollo.RegisterEventHandler("NextFrame",                        "OnFrame", self)
    Apollo.RegisterEventHandler("AccountCurrencyChanged",           "OnPlayerCurrencyChanged", self)
    Apollo.RegisterEventHandler("PlayerCurrencyChanged",            "OnPlayerCurrencyChanged", self)
    Apollo.RegisterEventHandler("PlayerLevelChange", 			    "OnXpChanged", self)
    Apollo.RegisterEventHandler("UI_XPChanged", 				    "OnXpChanged", self)
	Apollo.RegisterEventHandler("UpdatePathXp", 				    "OnXpChanged", self)
	Apollo.RegisterEventHandler("ElderPointsGained", 			    "OnXpChanged", self)
    Apollo.RegisterEventHandler("ChangeWorld", 					    "OnXpChanged", self)

    -- Misc
    Apollo.RegisterEventHandler("InterfaceMenu_ToggleShop",         "OnToggleShop", self)
    Apollo.RegisterEventHandler("InterfaceMenu_ToggleFortunes",     "OnToggleFortunes", self)
    Apollo.RegisterEventHandler("InterfaceMenuList_NewAddOn",       "OnNewAddonListed", self)
    Apollo.RegisterEventHandler("InterfaceMenuList_AlertAddOn",     "OnDrawAlert", self)
    Apollo.RegisterEventHandler("Tutorial_RequestUIAnchor",         "OnTutorial_RequestUIAnchor", self)
    Apollo.RegisterEventHandler("ApplicationWindowSizeChanged",     "ButtonListRedraw", self)
    Apollo.RegisterEventHandler("OptionsUpdated_HUDPreferences",    "OnOneSecTimer", self)
    Apollo.RegisterEventHandler("LiveEvent_WindowClosed",           "OnLiveEventClosed", self)
    Apollo.RegisterEventHandler("PublicEventStart",                 "OnPublicEventStart", self)
    Apollo.RegisterEventHandler("PublicEventEnd",                   "OnPublicEventEnd", self)
    Apollo.RegisterEventHandler("BonusEventsChanged",               "UpdateBonusEvents", self)

    self.bDocLoaded = true
end

function LUI_Frames:Build()
    self:LoadFrames()
    self:LoadMenuBar()
    self:LoadInfoBar()

    self.isBuild = true
end

function LUI_Frames:LoadFrames()
    if self.frames ~= nil then
        self.frames:Destroy()
        self.frames = nil
    end

    self.frames = Apollo.LoadForm(self.xmlDoc, "LUI_Frames", self.strata[self.settings.strata] or nil, self)

    -- Spacing
    self.frames:SetAnchorOffsets(self.settings.spacing,self.settings.spacing,-self.settings.spacing,-self.settings.spacing)

    -- Lines
    self.frames:FindChild("line_bottomleft"):SetAnchorOffsets(0,-self.settings.thickness,self.settings.chat.length_horizontal,0)
    self.frames:FindChild("line_bottomright"):SetAnchorOffsets(-self.settings.info.length_horizontal,-self.settings.thickness,0,0)
    self.frames:FindChild("line_left"):SetAnchorOffsets(0,-self.settings.chat.length_vertical,self.settings.thickness,0)
    self.frames:FindChild("line_right"):SetAnchorOffsets(-self.settings.thickness,-self.settings.info.length_vertical,0,0)
    self.frames:FindChild("line_centerleft"):SetAnchorOffsets(-self.settings.actionbar.line_length,-self.settings.thickness,self.settings.actionbar.elevate and -self.settings.actionbar.line_margin or 0,0)
    self.frames:FindChild("line_centeright"):SetAnchorOffsets(self.settings.actionbar.elevate and self.settings.actionbar.line_margin or 0,-self.settings.thickness,self.settings.actionbar.line_length,0)
    self.frames:FindChild("minimap_right"):SetAnchorOffsets(-self.settings.thickness,0,0,self.settings.minimap.length_vertical)
    self.frames:FindChild("minimap_top"):SetAnchorOffsets(-self.settings.minimap.length_horizontal,0,0,self.settings.thickness)

    self.frames:FindChild("line_bottomleft"):Show(self.settings.chat.showLines,true)
    self.frames:FindChild("line_bottomright"):Show(self.settings.info.showLines,true)
    self.frames:FindChild("line_left"):Show(self.settings.chat.showLines,true)
    self.frames:FindChild("line_right"):Show(self.settings.info.showLines,true)
    self.frames:FindChild("line_centerleft"):Show(self.settings.actionbar.showLines,true)
    self.frames:FindChild("line_centeright"):Show(self.settings.actionbar.showLines,true)
    self.frames:FindChild("minimap_right"):Show(self.settings.minimap.showLines,true)
    self.frames:FindChild("minimap_top"):Show(self.settings.minimap.showLines,true)

    -- Cubesize
    local half = (self.settings.cubeSize - self.settings.thickness) / 2
    self.frames:FindChild("line_bottomleft"):FindChild("bullet_right"):SetAnchorOffsets(0,-half,self.settings.cubeSize,half)
    self.frames:FindChild("line_bottomright"):FindChild("bullet_left"):SetAnchorOffsets(-self.settings.cubeSize,-half,0,half)
    self.frames:FindChild("line_left"):FindChild("bullet_top"):SetAnchorOffsets(-half,-self.settings.cubeSize,half,0)
    self.frames:FindChild("line_right"):FindChild("bullet_top"):SetAnchorOffsets(-half,-self.settings.cubeSize,half,0)
    self.frames:FindChild("line_centerleft"):FindChild("bullet_left"):SetAnchorOffsets(-self.settings.cubeSize,-half,0,half)
    self.frames:FindChild("line_centeright"):FindChild("bullet_right"):SetAnchorOffsets(0,-half,self.settings.cubeSize,half)
    self.frames:FindChild("minimap_right"):FindChild("bullet_bottom"):SetAnchorOffsets(-half,0,half,self.settings.cubeSize)
    self.frames:FindChild("minimap_top"):FindChild("bullet_left"):SetAnchorOffsets(-self.settings.cubeSize,-half,0,half)

    -- Connectors
    self.frames:FindChild("minimap_right"):FindChild("connector"):Show(self.settings.minimap.showConnectors,true)
    self.frames:FindChild("minimap_right"):FindChild("connector"):SetAnchorOffsets(
        self.settings.minimap.posX -self.settings.minimap.width / 2,
        self.settings.minimap.posY + (self.settings.minimap.height / 2),
        0,
        self.settings.minimap.posY + (self.settings.minimap.height / 2) + self.settings.thickness
    )

    self.frames:FindChild("minimap_top"):FindChild("connector"):Show(self.settings.minimap.showConnectors,true)
    self.frames:FindChild("minimap_top"):FindChild("connector"):SetAnchorOffsets(
        self.settings.minimap.posX - (self.settings.minimap.width / 2) - self.settings.thickness,
        0,
        self.settings.minimap.posX - (self.settings.minimap.width / 2),
        self.settings.minimap.posY + self.settings.minimap.height / 2
    )

    -- Minimap
    self.frames:FindChild("minimap"):Show(self.settings.minimap.enable,true)
    self.frames:FindChild("minimap"):SetAnchorOffsets(
        self.settings.minimap.posX - self.settings.minimap.width,
        self.settings.minimap.posY,
        self.settings.minimap.posX,
        self.settings.minimap.posY + self.settings.minimap.height
    )

    -- Sidebar
    self.frames:FindChild("sidebar"):Show(self.settings.sidebar.enable,true)
    self.frames:FindChild("sidebar"):SetAnchorOffsets(
        self.settings.sidebar.posX,
        self.settings.sidebar.posY,
        self.settings.sidebar.posX + self.settings.sidebar.width,
        self.settings.sidebar.posY + self.settings.sidebar.height
    )

    -- Actionbar
    self.frames:FindChild("actionbar"):Show(self.settings.actionbar.enable,true)
    self.frames:FindChild("actionbar"):SetAnchorOffsets(
        self.settings.actionbar.posX,
        self.settings.actionbar.posY,
        self.settings.actionbar.posX + self.settings.actionbar.width,
        self.settings.actionbar.posY + self.settings.actionbar.height
    )

    -- Elevator Left
    self.frames:FindChild("line_centerleft"):FindChild("line_up"):Show(self.settings.actionbar.elevate,true)
    self.frames:FindChild("line_centerleft"):FindChild("line_up"):SetAnchorOffsets(-self.settings.thickness,-self.settings.actionbar.line_up,0,0)
    self.frames:FindChild("line_centerleft"):FindChild("line_right"):SetAnchorOffsets(0,0,self.settings.actionbar.line_side,self.settings.thickness)

    -- Elevator Right
    self.frames:FindChild("line_centeright"):FindChild("line_up"):Show(self.settings.actionbar.elevate,true)
    self.frames:FindChild("line_centeright"):FindChild("line_up"):SetAnchorOffsets(0,-self.settings.actionbar.line_up,self.settings.thickness,0)
    self.frames:FindChild("line_centeright"):FindChild("line_left"):SetAnchorOffsets(-self.settings.actionbar.line_side,0,0,self.settings.thickness)

    self:ScaleAll(self.frames)
end

function LUI_Frames:OnFrame()
    if not self.bDocLoaded then
        return
    end

    if not self.unitPlayer or not self.unitPlayer:IsValid() then
        self:OnCharacterCreated()
        return
    end

    if not self.isBuild then
        self:Build()
        return
    end

    if self.lastCheck and ((Apollo.GetTickCount() - self.lastCheck) < self.settings.interval) then
        return
    end

    self.lastCheck = Apollo.GetTickCount()

    if self.shadow and self.shadow.wndShadow then
        if self.shadow.wndShadow:IsShown() ~= self.settings.showShadow then
            self.shadow.wndShadow:Show(self.settings.showShadow,true)
        end
    end

    local framesPerSecond = self:Round(GameLib.GetFrameRate(), 0)
    local latency = GameLib.GetLatency()

    self.infobar:FindChild("FPS"):SetText(framesPerSecond .. "fps")
    self.infobar:FindChild("MS"):SetText(latency .. "ms")
end

function LUI_Frames:OnXpChanged()
    if not self.unitPlayer or not self.unitPlayer:IsValid() then
        self:OnCharacterCreated()
        return
    end

    if self.unitPlayer:GetLevel() == 50 then
        self.currentXP = GetPeriodicElderPoints()
        self.neededXP = GameLib.ElderPointsDailyMax
        self.convertionXP = GameLib.ElderPointsPerGem

        self.infobar:FindChild("XP"):SetText("EG: "..self:Round(self.currentXP / self.neededXP * 100, 1) .. "%")
        self.infobar:FindChild("XP"):SetTooltip("EG: " .. math.floor(self.currentXP / self.convertionXP).."/"..math.floor(self.neededXP / self.convertionXP))
    else
        self.currentXP = GetXp() - GetXpToCurrentLevel()
        self.neededXP = GetXpToNextLevel()
        self.restedXP = GetRestXp()

        self.infobar:FindChild("XP"):SetText("XP: "..self:Round(self.currentXP / self.neededXP * 100, 1) .. "%")
        self.infobar:FindChild("XP"):SetTooltip("XP: " .. self:ShortNum(self.currentXP) .. "/" .. self:ShortNum(self.neededXP) .. " - rested: " .. self:ShortNum(self.restedXP) , "ffffffff", "CRB_InterfaceMedium")
    end
end

-- #########################################################################################################################################
-- # INFOBAR
-- #########################################################################################################################################

function LUI_Frames:LoadInfoBar()
    if self.infobar ~= nil then
        self.infobar:Destroy()
        self.infobar = nil
    end

    self.infobar = Apollo.LoadForm(self.xmlDoc, "LUI_Infobar", self.strata[self.settings.info.strata] or nil, self)
    self.currency = Apollo.LoadForm(self.xmlDoc, "OptionsConfigureCurrency", self.infobar, self)
    self.currencyList = self.currency:FindChild("OptionsConfigureCurrencyList")
    self.infobar:FindChild("OptionsBtn"):AttachWindow(self.currency)

    self.infobar:SetAnchorOffsets(
        self.settings.info.posX - 700,
        self.settings.info.posY,
        self.settings.info.posX,
        self.settings.info.posY + 32
    )

    for idx = 1, #karCurrency do
        local tData = karCurrency[idx]
        local wnd
        wnd = Apollo.LoadForm(self.xmlDoc, "PickerEntry", self.currencyList, self)

        if tData.bAccountItem then
            wnd:FindChild("EntryCash"):SetMoneySystem(Money.CodeEnumCurrencyType.GroupCurrency, 0, 0, tData.eType)
        else
            wnd:FindChild("EntryCash"):SetMoneySystem(tData.eType)
        end

        wnd:FindChild("PickerEntryBtn"):SetData(idx)
        wnd:FindChild("PickerEntryBtn"):SetCheck(idx == self.settings.currency)
        wnd:FindChild("PickerEntryBtn"):SetText(tData.strTitle)
        tData.wnd = wnd
    end

    local oLeft,_,oRight,oBottom = self.currency:GetAnchorOffsets()
    local nHeight = ((#karCurrency * 29) + 12 + 4) * -1

    self.currency:SetAnchorOffsets(oLeft,nHeight,oRight,oBottom)
    self.currencyList:ArrangeChildrenVert(Window.CodeEnumArrangeOrigin.LeftOrTop)

    if self.settings.info.showTime then
        self.infobar:FindChild("Gold"):SetAnchorOffsets(-250,0,-100,0)
        self.infobar:FindChild("Gold"):FindChild("BG"):SetAnchorOffsets(1,1,0,-1)
        self.infobar:FindChild("Time"):SetText(self:GetTime())
        self:StartTime()
    else
        self.infobar:FindChild("Gold"):SetAnchorOffsets(-150,0,0,0)
    end

    self.infobar:FindChild("Time"):Show(self.settings.info.showTime,true)
    self.infobar:FindChild("XP"):Show(self.settings.info.showXP,true)

    self:ScaleAll(self.infobar)
    self:UpdateAltCashDisplay()
    self:OnXpChanged()
end

function LUI_Frames:StartTime()
    if not self.timerCreateDelay then
        self.timerCreateDelay = ApolloTimer.Create(1.0, true, "OnOneSecTimer", self)
    else
        self.timerCreateDelay:Stop()
        self.timerCreateDelay:Start()
    end
end

function LUI_Frames:OnOneSecTimer()
    if self.settings.info.showTime then
        self.infobar:FindChild("Time"):SetText(self:GetTime())
    end

    if not self.bHasLoaded then
        if not self.wndMain then
            return
        end

        Event_FireGenericEvent("InterfaceMenuListHasLoaded")
        self.wndMain:FindChild("OpenFullListBtn"):Enable(true)
        self.bHasLoaded = true
    end
end

function LUI_Frames:GetTime()
    local l_time = GameLib.GetLocalTime()

    if self.settings.info.b24HourFormat then
        return string.format("%02d:%02d", l_time.nHour, l_time.nMinute)
    else
        if l_time.nHour > 12 then
            return string.format("%02d:%02d", l_time.nHour - 12, l_time.nMinute)
        else
            if l_time.nHour == 0 then
                return string.format("%02d:%02d", l_time.nHour + 12, l_time.nMinute)
            else
                return string.format("%02d:%02d", l_time.nHour, l_time.nMinute)
            end
        end
    end
end

function LUI_Frames:UpdateAltCash(wndHandler, wndControl)
    local nSelected = wndHandler:FindChild("PickerEntryBtn"):GetData()
    local tData = karCurrency[nSelected]

    if wndHandler:FindChild("PickerEntryBtn"):IsChecked() then
        self.settings.currency = nSelected
    end

    self:UpdateAltCashDisplay()
    tData.wnd:FindChild("EntryCash"):SetAmount(self:HelperGetCurrencyAmmount(tData), true)

    if self.currency:IsShown() then
        self.currency:Show(false,true)
    end
end

function LUI_Frames:UpdateAltCashDisplay()
    local tData = karCurrency[self.settings.currency]

    if not tData or not self.infobar then
        return
    end

    local currency = GameLib.GetPlayerCurrency()

    if currency then
        local amount = tostring(currency:GetAmount())
        local len = string.len(amount)

        if len and len >= 7 then
            currency:SetAmount(tonumber(string.sub(amount,0,len-4).."0000"))
        end
    end

    self.infobar:FindChild("AltCashWindow"):SetAmount(self:HelperGetCurrencyAmmount(tData), true)
    self.infobar:FindChild("MainCashWindow"):SetAmount(currency, true)

    if karCurrency[self.settings.currency] and karCurrency[self.settings.currency].strTitle == "OmniBits" then
        self.omnibits = GameLib.GetOmnibitsBonusInfo()
        self.infobar:FindChild("Currency"):FindChild("OptionsBtn"):SetTooltip(tostring(self.omnibits.nWeeklyBonusEarned).."/"..tostring(self.omnibits.nWeeklyBonusMax))
    else
        self.infobar:FindChild("Currency"):FindChild("OptionsBtn"):SetTooltip("")
    end

    self:CheckMainCashWidth()
    self:CheckAltCashWidth()
end

function LUI_Frames:CheckMainCashWidth()
    local nLeft = (self.settings.info.showTime) and 160 or 60
    local nRight = (self.settings.info.showTime) and 100 or 0

    local amount = tostring(self.infobar:FindChild("MainCashWindow"):GetAmount())
    local nCurrencyWidth = Apollo.GetTextWidth("CRB_Header9", amount) + 60
    local len = string.len(amount)

    if len and len >= 7 then
        nCurrencyWidth = Apollo.GetTextWidth("CRB_Header9", string.sub(amount,0,len-4)) + 40
    end

    local nNewWidth = (nLeft+nCurrencyWidth)*-1
    self.infobar:FindChild("Gold"):SetAnchorOffsets(self:GetDimension(nNewWidth),0,self:GetDimension(nRight),0)
end

function LUI_Frames:CheckAltCashWidth()
    local nCurrencyWidth = -(self.infobar:FindChild("AltCashWindow"):GetDisplayWidth() + 40)
    self.infobar:FindChild("Currency"):SetAnchorOffsets(self:GetDimension(nCurrencyWidth), 0, 0, 0)
end

function LUI_Frames:OnCurrencyPanelToggle(wndHandler, wndControl)
    for key, wndCurr in pairs(self.currencyList:GetChildren()) do
        self:UpdateAltCash(wndCurr)
    end
end

-- #########################################################################################################################################
-- # MENUBAR
-- #########################################################################################################################################

function LUI_Frames:LoadMenuBar()
    if self.wndMain ~= nil then
        self.wndMain:Destroy()
        self.wndMain = nil
    end

    if self.wndList ~= nil then
        self.wndList:Destroy()
        self.wndList = nil
    end

    self.wndMain = Apollo.LoadForm(self.xmlDoc , "LUI_Menubar", self.strata[self.settings.chat.strata] or nil, self)
    self.wndList = Apollo.LoadForm(self.xmlDoc , "FullListFrame", self.wndMain, self)

    self.wndMain:FindChild("OpenFullListBtn"):AttachWindow(self.wndList)
    self.wndMain:FindChild("OpenFullListBtn"):Enable(false)
    self.bHasLoaded = nil

    self.wndMain:FindChild("Store"):Show(self.settings.chat.showStore,true)
    self.wndMain:FindChild("MTX"):Show(self.settings.chat.showMadameFay,true)

    self.wndMain:SetAnchorOffsets(
        self.settings.chat.posX,
        self.settings.chat.posY,
        self.settings.chat.posX + 700,
        self.settings.chat.posY + 32
    )

    if not self.tWidth then
        self.tWidth = {
            ["menu"] = self.wndMain:FindChild("Menu"):GetWidth(),
            ["store"] = self.wndMain:FindChild("Store"):GetWidth() - 1,
            ["mtx"] = self.wndMain:FindChild("MTX"):GetWidth(),
            ["event"] = self.wndMain:FindChild("LiveEvent"):GetWidth()
        }
    end

    local nLeft,nTop,nRight,nBottom = self.wndMain:FindChild("ButtonList"):GetAnchorOffsets() --wndButtonList:GetOriginalLocation():GetOffsets()

    if not self.tOffsets then
        self.offsets = {
            ["left"] = nLeft,
            ["top"] = nTop,
            ["right"] = nRight,
            ["bottom"] = nBottom
        }
    end

    if not self.tMenuData then
        self.tMenuData = {
            [Apollo.GetString("InterfaceMenu_SystemMenu")] = { "", "Escape", "Icon_Windows32_UI_CRB_InterfaceMenu_EscMenu" },
            [Apollo.GetString("InterfaceMenu_Store")] = { "", "Store", "Icon_Windows32_UI_CRB_InterfaceMenu_Credd" },
            [Apollo.GetString("InterfaceMenuList_Fortunes")] = { "InterfaceMenu_ToggleFortunes", "", "" },
        }
        self.tMenuTooltips = {}
        self.tMenuAlerts = {}
    end

    self.nPrevNumLiveEvents = 0
    self.nNumLiveEvents = 0
    if self.tLiveEvents == nil then
        self.tLiveEvents = {}
    end
    self.bPreviouslyHadBonusEvents = false
    self.bHasBonusEvents = false

    --Event_FireGenericEvent("InterfaceMenuList_NewAddOn", Apollo.GetString("InterfaceMenu_Store"), {"InterfaceMenu_ToggleShop", "Store", ""})
    --Event_FireGenericEvent("InterfaceMenuList_NewAddOn", Apollo.GetString("InterfaceMenuList_Fortunes"), {"InterfaceMenu_ToggleFortunes", "", ""})

    self:ScaleAll(self.wndMain)
    self:ButtonListRedraw()
    self:FullListRedraw()
end

function LUI_Frames:FullListRedraw()
    local strUnbound = Apollo.GetString("Keybinding_Unbound")
    local wndParent = self.wndList:FindChild("FullListScroll")
    wndParent:DestroyChildren()

    for strAddonName, tData in pairs(self.tMenuData) do
        local wndMenuItem = Apollo.LoadForm(self.xmlDoc, "MenuListItem", wndParent, self)
        local strTooltip = strAddonName
        wndMenuItem:SetName(strAddonName)
        self:ScaleAll(wndMenuItem)

        if string.len(tData[2]) > 0 then
            local strKeyBindLetter = GameLib.GetKeyBinding(tData[2])
            strKeyBindLetter = (strKeyBindLetter and strKeyBindLetter ~= strUnbound) and string.format(" (%s)", strKeyBindLetter) or ""
            strTooltip = strTooltip .. strKeyBindLetter
        end

        if tData[3] ~= "" then
            wndMenuItem:FindChild("Icon"):SetSprite(tData[3])
        else
            wndMenuItem:FindChild("Icon"):SetText(string.sub(strTooltip, 1, 1))
        end

        wndMenuItem:FindChild("IconContainer"):SetData(strAddonName)
        wndMenuItem:FindChild("Icon"):SetTooltip(strTooltip)
        self.tMenuTooltips[strAddonName] = strTooltip

        wndMenuItem:FindChild("MenuListItemBtn"):FindChild("MenuListItemName"):SetText(strAddonName)
        wndMenuItem:FindChild("MenuListItemBtn"):SetData(strAddonName)

        wndMenuItem:FindChild("PinBtn"):SetCheck(self:IsPinned(strAddonName))
        wndMenuItem:FindChild("PinBtn"):SetData(strAddonName)

        if string.len(tData[2]) > 0 then
            local strKeyBindLetter = GameLib.GetKeyBinding(tData[2])
            wndMenuItem:FindChild("MenuListItemBtn"):FindChild("MenuListItemKeybind"):SetText((strKeyBindLetter and strKeyBindLetter ~= strUnbound) and string.format("(%s)", strKeyBindLetter) or "")
        end

        if self.tMenuAlerts[strAddonName] then
            self:OnDrawAlertVisual(strAddonName, self.tMenuAlerts[strAddonName])
        end
    end

    wndParent:ArrangeChildrenVert(Window.CodeEnumArrangeOrigin.LeftOrTop, function (a,b) return a:GetName() < b:GetName() end)
end

function LUI_Frames:ButtonListRedraw(time)
    local delay = time or 0.3

    if not self.QueueRedrawTimer then
        self.QueueRedrawTimer = ApolloTimer.Create(delay, false, "OnQueuedRedraw", self)
    else
        self.QueueRedrawTimer:Stop()
        self.QueueRedrawTimer:Start()
    end
end

function LUI_Frames:OnQueuedRedraw()
    if not self.wndMain then
        return
    end

    local wndParent = self.wndMain:FindChild("ButtonList")
    wndParent:DestroyChildren()

    for idx, strPinName in ipairs(self.settings.tPinnedAddons) do
        self:AddPinToButtonList(strPinName)
    end

    for _,button in pairs(wndParent:GetChildren()) do
        self:ScaleAll(button)
    end

    if not self.settings.chat.showEvent then
        self.wndMain:FindChild("BonusEvent"):Show(false,true)
        self.wndMain:FindChild("LiveEvent"):Show(false,true)
    end

    self.wndMain:FindChild("Store"):Show(self.settings.chat.showStore,true)
    self.wndMain:FindChild("MTX"):Show(self.settings.chat.showMadameFay,true)

    local offsetMTX = self.tWidth["menu"]
    local offsetLiveEvent = self.tWidth["menu"]
    local offsetBonusEvent = self.tWidth["menu"]
    local offsetButtonList = self.tWidth["menu"]

    if self.wndMain:FindChild("Store"):IsShown() then
        offsetMTX = offsetMTX + self.tWidth["store"]
        offsetLiveEvent = offsetLiveEvent + self.tWidth["store"]
        offsetBonusEvent = offsetBonusEvent + self.tWidth["store"]
        offsetButtonList = offsetButtonList + self.tWidth["store"]
    end

    if self.wndMain:FindChild("MTX"):IsShown() then
        offsetLiveEvent = offsetLiveEvent + self.tWidth["mtx"]
        offsetBonusEvent = offsetBonusEvent + self.tWidth["mtx"]
        offsetButtonList = offsetButtonList + self.tWidth["mtx"]
    end

    if self.wndMain:FindChild("LiveEvent"):IsShown() then
        offsetBonusEvent = offsetBonusEvent + self.tWidth["event"]
        offsetButtonList = offsetButtonList + self.tWidth["event"]
    end

    if self.wndMain:FindChild("BonusEvent"):IsShown() then
        offsetButtonList = offsetButtonList + self.tWidth["event"]
    end

    self.wndMain:FindChild("MTX"):SetAnchorOffsets(
        self:GetDimension(offsetMTX),
        self:GetDimension(self.offsets["top"]),
        self:GetDimension(offsetMTX+self.tWidth["mtx"]),
        self:GetDimension(self.offsets["bottom"])
    )

    self.wndMain:FindChild("LiveEvent"):SetAnchorOffsets(
        self:GetDimension(offsetLiveEvent),
        self:GetDimension(self.offsets["top"]),
        self:GetDimension(offsetLiveEvent+self.tWidth["event"]),
        self:GetDimension(self.offsets["bottom"])
    )

    self.wndMain:FindChild("BonusEvent"):SetAnchorOffsets(
        self:GetDimension(offsetBonusEvent),
        self:GetDimension(self.offsets["top"]),
        self:GetDimension(offsetBonusEvent+self.tWidth["event"]),
        self:GetDimension(self.offsets["bottom"])
    )

    self.wndMain:FindChild("ButtonList"):SetAnchorOffsets(
        self:GetDimension(offsetButtonList),
        self:GetDimension(self.offsets["top"]),
        self:GetDimension(self.offsets["right"]),
        self:GetDimension(self.offsets["bottom"])
    )
end

function LUI_Frames:AddPinToButtonList(strAddonName)
    if not self.wndMain then
        return
    end

    local strUnbound = Apollo.GetString("Keybinding_Unbound")
    local wndParent = self.wndMain:FindChild("ButtonList")
    -- local nParentWidth = wndParent:GetWidth()
    local tData = self.tMenuData[strAddonName]

    if tData then
        local wndMenuItem = Apollo.LoadForm(self.xmlDoc , "InterfaceMenuButton", wndParent, self)
        local strTooltip = strAddonName
        wndMenuItem:SetName(strAddonName)

        if string.len(tData[2]) > 0 then
            local strKeyBindLetter = GameLib.GetKeyBinding(tData[2])
            strKeyBindLetter = (strKeyBindLetter and strKeyBindLetter ~= strUnbound) and string.format(" (%s)", strKeyBindLetter) or ""
            strTooltip = strTooltip .. strKeyBindLetter
        end

        if tData[3] ~= "" then
            wndMenuItem:FindChild("Icon"):SetSprite(tData[3])
        else
            wndMenuItem:FindChild("Icon"):SetText(string.sub(strTooltip, 1, 1))
        end

        wndMenuItem:FindChild("ShortcutBtn"):SetData(strAddonName)
        wndMenuItem:FindChild("Icon"):SetTooltip(strTooltip)
        wndMenuItem:SetName("InterfaceMenuButton_" .. strAddonName)
    end

    wndParent:ArrangeChildrenHorz(Window.CodeEnumArrangeOrigin.LeftOrTop)

    if self.tMenuAlerts[strAddonName] then
        self:OnDrawAlertVisual(strAddonName, self.tMenuAlerts[strAddonName])
    end
end

function LUI_Frames:OnNewAddonListed(strKey, tParams)
    if type(strKey) ~= "string" or type(tParams) ~= "table" then
        return
    end

    strKey = string.gsub(strKey, ":", "|")

    self.tMenuData[strKey] = tParams
    self:ButtonListRedraw()
end

function LUI_Frames:RemovePinFromButtonList(strAddonName)
    if not self.wndMain then
        return
    end

    local wndParent = self.wndMain:FindChild("ButtonList")
    local wndButtonListItem = wndParent:FindChild("InterfaceMenuButton_" .. strAddonName)

    if wndButtonListItem then
        wndButtonListItem:Destroy()
        wndParent:ArrangeChildrenHorz(Window.CodeEnumArrangeOrigin.LeftOrTop)
    end
end

function LUI_Frames:OnPinChecked(wndHandler, wndControl)
    local strPinName = wndControl:GetData()
    -- local nPinTableSize = self:GetTableSize(self.settings.tPinnedAddons) + 1

    if not self:GetPinIndex(self.settings.tPinnedAddons, strPinName) then
        table.insert(self.settings.tPinnedAddons, strPinName)
        self:OnQueuedRedraw()
    end
end

function LUI_Frames:OnPinUnchecked(wndHandler, wndControl)
    local strPinName = wndControl:GetData()
    local nPinIndex = self:GetPinIndex(self.settings.tPinnedAddons, strPinName)

    if nPinIndex > 0 then
        table.remove(self.settings.tPinnedAddons, nPinIndex)
    end

    self:RemovePinFromButtonList(strPinName)
end

function LUI_Frames:IsPinned(strText)
    local nHasTableValue = self:GetPinIndex(self.settings.tPinnedAddons,strText)
    if nHasTableValue ~= nil then
        return true
    end

    return false
end

function LUI_Frames:OnCharacterCreated()
    self.unitPlayer = GameLib.GetPlayerUnit()

    if self.unitPlayer and self.unitPlayer:IsValid() then
        -- Menubar
        self:StartTime()
        self:UpdateLiveEvents()
        self:UpdateBonusEvents()

        -- Infobar
        self:UpdateAltCashDisplay()

        -- Menubar
        self:ButtonListRedraw()
        self:OnXpChanged()

        if self.unitPlayerTimer then
            self.unitPlayerTimer:Stop()
            self.unitPlayerTimer = nil
        end
    else
        if not self.unitPlayerTimer then
            self.unitPlayerTimer = ApolloTimer.Create(0.1, false, "OnCharacterCreated", self)
        end

        self.unitPlayerTimer:Start()
    end
end

function LUI_Frames:OnToggleShop()
    GameLib.OpenStore()
end

function LUI_Frames:OnToggleFortunes()
    GameLib.OpenFortunes()
end

function LUI_Frames:OnListShow()
    self.wndList:ToFront()
end

function LUI_Frames:OnListClose()
    if not self.wndMain then
        return
    end

    self.wndMain:FindChild("OpenFullListBtn"):SetCheck(false)
end

-----------------------------------------------------------------------------------------------
-- Events
-----------------------------------------------------------------------------------------------

function LUI_Frames:OnPublicEventStart(peStart)
    local nId
    local leEvent = peStart:GetLiveEvent()
    if leEvent then
        nId = leEvent:GetId()
    else
        return
    end

    if peStart:GetEventType() == PublicEvent.PublicEventType_LiveEvent then
        if self.tLiveEvents == nil then
            self.tLiveEvents = {}
        end
        if not self.tLiveEvents[nId] then
            self.tLiveEvents[nId] = true
            self.nNumLiveEvents = self.nNumLiveEvents + 1
        end

        self:UpdateLiveEventButtons()
    end
end

function LUI_Frames:OnPublicEventEnd(peEnd)
    local nId
    local leEvent = peEnd:GetLiveEvent()
    if leEvent then
        nId = leEvent:GetId()
    else
        return
    end

    if peEnd:GetEventType() == PublicEvent.PublicEventType_LiveEvent then
        if self.tLiveEvents[nId] then
            self.tLiveEvents[nId] = nil
            self.nNumLiveEvents = self.nNumLiveEvents - 1
        end

        self:UpdateLiveEventButtons()
    end
end

function LUI_Frames:UpdateLiveEvents()
    self.nNumLiveEvents = 0
    self.tLiveEvents = {}

    for idx, peEvent in pairs(LiveEventsLib.GetActiveLiveEvents()) do
        self.tLiveEvents[peEvent:GetLiveEvent():GetId()] = true
        self.nNumLiveEvents = self.nNumLiveEvents + 1
    end

    self:UpdateLiveEventButtons()
end

function LUI_Frames:UpdateBonusEvents()
    self.bHasBonusEvents = false
    for idx, leEvent in pairs(LiveEventsLib.GetBonusLiveEventList()) do
        self.bHasBonusEvents = true
        break
    end

    self:UpdateLiveEventButtons()
end

function LUI_Frames:UpdateLiveEventButtons()
    if not self.wndMain then
        return
    end

    local bButtonUpdated = false
    if self.nPrevNumLiveEvents < self.nNumLiveEvents then
        if self.nPrevNumLiveEvents <= 0 then
            local leEvent = nil

            for nLiveEventId, bActive in pairs(self.tLiveEvents) do
                leEvent = LiveEventsLib.GetLiveEvent(nLiveEventId)
                break
            end

            if leEvent == nil then
                return
            end

            local wndLiveEvent = self.wndMain:FindChild("LiveEvent")
            wndLiveEvent:Show(true)
            self.wndMain:FindChild("BonusEvent"):Show(false)
            bButtonUpdated = true

            local wndLiveEventBtn = wndLiveEvent:FindChild("EventMoreInfoBtn")
            wndLiveEventBtn:FindChild("Icon"):SetSprite(leEvent:GetButtonIcon() ~= "" and leEvent:GetButtonIcon() or leEvent:GetIcon())
            wndLiveEventBtn:SetTooltip(string.format(
                "%s%s%s",
                Apollo.GetString("Event_WorldEventTitle"),
                Apollo.GetString("Chat_ColonBreak"),
                leEvent:GetName()
            ))

            Event_FireGenericEvent("LiveEvent_OpenWindow")
        end

        self.nPrevNumLiveEvents = self.nNumLiveEvents
    elseif self.nPrevNumLiveEvents > self.nNumLiveEvents then
        if self.nNumLiveEvents <= 0 then
            local wndLiveEvent = self.wndMain:FindChild("LiveEvent")
            wndLiveEvent:Show(false)
            self.wndMain:FindChild("BonusEvent"):Show(self.bHasBonusEvents)
            bButtonUpdated = true

            Event_FireGenericEvent("LiveEvent_CloseWindow")
        end

        self.nPrevNumLiveEvents = self.nNumLiveEvents
    end

    if not self.bPreviouslyHadBonusEvents and self.bHasBonusEvents then
        local wndBonusEvent = self.wndMain:FindChild("BonusEvent")
        wndBonusEvent:Show(not self.wndMain:FindChild("LiveEvent"):IsShown())
        bButtonUpdated = true

        self.bPreviouslyHadBonusEvents = self.bHasBonusEvents
    elseif self.bPreviouslyHadBonusEvents and not self.bHasBonusEvents then
        local wndBonusEvent = self.wndMain:FindChild("BonusEvent")
        wndBonusEvent:Show(false)
        bButtonUpdated = true

        self.bPreviouslyHadBonusEvents = self.bHasBonusEvents
    end

    if not self.settings.chat.showEvent then
        self.wndMain:FindChild("BonusEvent"):Show(false,true)
        self.wndMain:FindChild("LiveEvent"):Show(false,true)
    end

    if bButtonUpdated then
        --self:LoadMenuBar()
        self:ButtonListRedraw()
    end
end

-----------------------------------------------------------------------------------------------
-- Alerts
-----------------------------------------------------------------------------------------------

function LUI_Frames:OnDrawAlert(strAddonName, tParams)
    if not strAddonName or not tParams or type(strAddonName) ~= "string" or type(tParams) ~= "table" then
        return
    end

    if not self.wndMain then
        return
    end

    local wndButtonList = self.wndMain:FindChild("ButtonList")
    local wndButtonListItem = wndButtonList:FindChild("InterfaceMenuButton_" .. strAddonName)
    local nPinIndex = self:GetPinIndex(self.settings.tPinnedAddons, strAddonName)

    if wndButtonListItem and nPinIndex ~= nil then
        local wndButton = wndButtonListItem:FindChild("ShortcutBtn")
        if tParams[3] and self.tMenuAlerts[strAddonName] and self.tMenuAlerts[strAddonName][3] < tParams[3] then

            if wndButton:FindChild("AlertBlip") then
                wndButton:FindChild("AlertBlip"):Destroy()
            end

            local wndFlash = Apollo.LoadForm(self.xmlDoc , "AlertBlip", wndButton:FindChild("Alert"), self)
            wndFlash:SetName("AlertBlip")
            wndFlash:FindChild("Sonar"):SetSprite("PlayerPathContent_TEMP:spr_Crafting_TEMP_Stretch_QuestZoneNoLoop")
        elseif wndButton:FindChild("AlertBlip") ~= nil then
            wndButton:FindChild("AlertBlip"):Destroy()
        end
    end

    self.tMenuAlerts[strAddonName] = tParams
    self:OnDrawAlertVisual(strAddonName, tParams)
end

function LUI_Frames:OnDrawAlertVisual(strAddonName, tParams)
    if type(strAddonName) ~= "string" or type(tParams) ~= "table" then
        return
    end

    if not self.wndMain then
        return
    end

    local wndButtonList = self.wndMain:FindChild("ButtonList")
    local wndButtonListItem = wndButtonList:FindChild("InterfaceMenuButton_" .. strAddonName)
    local nPinIndex = self:GetPinIndex(self.settings.tPinnedAddons, strAddonName)

    if wndButtonListItem and nPinIndex ~= nil then
        local wndButton = wndButtonListItem:FindChild("ShortcutBtn")
        local wndIcon = wndButton:FindChild("Icon")

        if tParams[1] then
            if wndButton:FindChild("AlertIndicator") then
                wndButton:FindChild("AlertIndicator"):Destroy()
            end

            local wndIndicator = Apollo.LoadForm(self.xmlDoc , "AlertIndicator", wndButton:FindChild("Alert"), self)
            wndIndicator:SetName("AlertIndicator")
        elseif wndButton:FindChild("AlertIndicator") ~= nil then
            wndButton:FindChild("AlertIndicator"):Destroy()
        end

        if tParams[2] and self.tMenuTooltips[strAddonName] then
            wndIcon:SetTooltip(string.format("%s\n\n%s", self.tMenuTooltips[strAddonName], tParams[2]))
        end

        if tParams[3] and tParams[3] > 0 then
            local strColor = tParams[1] and "UI_WindowTextOrange" or "UI_TextHoloTitle"

            wndButton:FindChild("Number"):Show(true)
            wndButton:FindChild("Number"):SetText(tParams[3])
            wndButton:FindChild("Number"):SetTextColor(ApolloColor.new(strColor))
        else
            wndButton:FindChild("Number"):Show(false)
            wndButton:FindChild("Number"):SetText("")
            wndButton:FindChild("Number"):SetTextColor(ApolloColor.new("UI_TextHoloTitle"))
        end
    end

    local wndParent = self.wndList:FindChild("FullListScroll")
    for idx, wndTarget in pairs(wndParent:GetChildren()) do
        local wndButton = wndTarget:FindChild("IconContainer")
        local wndIcon = wndButton:FindChild("Icon")

        if wndButton:GetData() == strAddonName then
            if tParams[1] then
                local wndIndicator = Apollo.LoadForm(self.xmlDoc , "AlertIndicator", wndButton:FindChild("Alert"), self)
                wndIndicator:SetName("AlertIndicator")
            elseif wndButton:FindChild("AlertIndicator") ~= nil then
                wndButton:FindChild("AlertIndicator"):Destroy()
            end

            if tParams[2] and self.tMenuTooltips[strAddonName] then
                wndIcon:SetTooltip(string.format("%s\n\n%s", self.tMenuTooltips[strAddonName], tParams[2]))
            end

            if tParams[3] and tParams[3] > 0 then
                local strColor = tParams[1] and "UI_WindowTextOrange" or "UI_TextHoloTitle"

                wndButton:FindChild("Number"):Show(true)
                wndButton:FindChild("Number"):SetText(tParams[3])
                wndButton:FindChild("Number"):SetTextColor(ApolloColor.new(strColor))
            else
                wndButton:FindChild("Number"):Show(false)
            end
        end
    end
end

function LUI_Frames:OnLiveEventClosed()
end

-----------------------------------------------------------------------------------------------
-- Helpers and Errata
-----------------------------------------------------------------------------------------------

function LUI_Frames:OnMenuListItemClick(wndHandler, wndControl)
    if wndHandler ~= wndControl then return end

    local strKey = wndHandler:GetData()
    local strMappingResult = (strKey and self.tMenuData[strKey]) and self.tMenuData[strKey][1] or ""

    if string.len(strMappingResult) > 0 then
        Event_FireGenericEvent(strMappingResult)
    else
        if strKey == Apollo.GetString("InterfaceMenu_SystemMenu") then
            InvokeOptionsScreen()
        elseif strKey == Apollo.GetString("InterfaceMenu_Store") then
            GameLib.OpenStore()
        else
            InvokeOptionsScreen()
        end
    end

    self.wndList:Close()
end

function LUI_Frames:OnMenuListMouseEnter(wndHandler,wndControl)
    if wndControl:FindChild("MenuListItemName") then
        wndControl:FindChild("MenuListItemName"):SetTextColor("ffffffff")
    end
end

function LUI_Frames:OnMenuListMouseExit(wndHandler,wndControl)
    if wndControl:FindChild("MenuListItemName") then
        wndControl:FindChild("MenuListItemName"):SetTextColor("ffebebeb")
    end
end

function LUI_Frames:OnListBtnClick(wndHandler, wndControl)
    if wndHandler ~= wndControl then return end
    local strKey = wndHandler:GetData()
    local strMappingResult = wndHandler:GetData() and self.tMenuData[strKey][1] or ""

    if string.len(strMappingResult) > 0 then
        Event_FireGenericEvent(strMappingResult)
    else
        if strKey == Apollo.GetString("InterfaceMenu_SystemMenu") then
            InvokeOptionsScreen()
        elseif strKey == Apollo.GetString("InterfaceMenu_Store") then
            GameLib.OpenStore()
        else
            InvokeOptionsScreen()
        end
    end

    local wndAlertActive = wndControl:FindChild("AlertBlip") or nil
    if wndAlertActive ~= nil then
        wndAlertActive:Destroy()
    end
end

function LUI_Frames:OnOpenFullListCheck(wndHandler, wndControl)
    self.wndList:Invoke()
    self:FullListRedraw()
end

function LUI_Frames:OnPlayerCurrencyChanged()
    self:UpdateAltCashDisplay()
end

function LUI_Frames:OnEventMoreInfoBtn(wndHandler, wndControl)
    Event_FireGenericEvent("LiveEvent_ToggleWindow")
end

function LUI_Frames:OnLoginRewardsOpenBtn(wndHandler, wndControl)
    Event_FireGenericEvent("LiveEvent_ToggleWindow")
end

function LUI_Frames:OnTutorial_RequestUIAnchor(eAnchor, idTutorial, strPopupText)
    local arTutorialAnchorMapping =
    {
        --[GameLib.CodeEnumTutorialAnchor.Abilities]             = "LASBtn",
        --[GameLib.CodeEnumTutorialAnchor.Character]         = "CharacterBtn",
        --[GameLib.CodeEnumTutorialAnchor.Mail]                 = "MailBtn",
        --[GameLib.CodeEnumTutorialAnchor.GalacticArchive] = "LoreBtn",
        --[GameLib.CodeEnumTutorialAnchor.Social]             = "SocialBtn",
        --[GameLib.CodeEnumTutorialAnchor.GroupFinder]         = "GroupFinderBtn",
    }

    local strWindowName = "ButtonList" or false
    if not strWindowName then
        return
    end

    if not self.wndMain then
        return
    end

    local tRect = {}
    tRect.l, tRect.t, tRect.r, tRect.b = self.wndMain:FindChild(strWindowName):GetRect()
    tRect.r = tRect.r - 26

    if arTutorialAnchorMapping[eAnchor] then
        Event_FireGenericEvent("Tutorial_RequestUIAnchorResponse", eAnchor, idTutorial, strPopupText, tRect)
    end
end

function LUI_Frames:OnGatchaOpenBtn(wndHandler, wndControl)
    GameLib.OpenFortunes()
end

function LUI_Frames:OnMarketplaceOpenBtn(wndHandler, wndControl)
    GameLib.OpenStore()
end

function LUI_Frames:GetTableSize(tName)
    local nCount = 0
    for _ in pairs(tName) do
        nCount = nCount + 1
    end
    return nCount
end

function LUI_Frames:GetPinIndex(tName, strValue)
    for idx, strPinName in ipairs(tName) do
        if strPinName == strValue then
            return idx
        end
    end
    return nil
end

-- #########################################################################################################################################
-- # MISC
-- #########################################################################################################################################

function LUI_Frames:OnSave(eType)
    if eType ~= GameLib.CodeEnumAddonSaveLevel.Account then
        return
    end

    return self.settings
end

function LUI_Frames:OnRestore(eLevel, tSavedData)
    if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Account then
        return
    end

    if not tSavedData then
        return
    end

    self.settings = self:InsertDefaults(tSavedData, self.settings)
end

function LUI_Frames:InsertDefaults(t,defaults)
    for k,v in pairs(defaults) do
        if t[k] == nil then
            if type(v) == 'table' then
                t[k] = self:Copy(v)
            else
                t[k] = v
            end
        else
            if type(v) == 'table' then
                t[k] = self:InsertDefaults(t[k],v)
            end
        end
    end

    return t
end

function LUI_Frames:Copy(t)
    local o = {}

    for k,v in pairs(t) do
        if type(v) == 'table' then
            o[k] = self:Copy(v)
        else
            o[k] = v
        end
    end

    return o
end

function LUI_Frames:ShortNum(num)
    local tmp = tostring(num)
    local ret
    if not num then
        return 0
    elseif num >= 1000000 then
        ret = string.sub(tmp, 1, string.len(tmp) - 6) .. "." .. string.sub(tmp, string.len(tmp) - 5, string.len(tmp) - 5) .. "M"
    elseif num >= 1000 then
        ret = string.sub(tmp, 1, string.len(tmp) - 3) .. "." .. string.sub(tmp, string.len(tmp) - 2, string.len(tmp) - 2) .. "k"    else
        ret = num -- hundreds
    end
    return ret
end

function LUI_Frames:HelperGetCurrencyAmmount(tData)
    local monAmount
    if tData.bAccountItem then
        monAmount = AccountItemLib.GetAccountCurrency(tData.eType) or 0
    else
        monAmount = GameLib.GetPlayerCurrency(tData.eType) or 0
    end
    return monAmount
end

function LUI_Frames:GetDimension(x)
    -- local scale = Apollo.GetConsoleVariable("ui.Scale") or 1
    local size = Apollo.GetDisplaySize()
    local mult = size.nHeight / size.nRawHeight
    ---return x

    if self.settings.useUIScale then
        return x
    else
        return mult * math.floor(x + .5)
    end
end

function LUI_Frames:Scale(wnd)
    local nLeft, nTop, nRight, nBottom = wnd:GetAnchorOffsets()
    wnd:SetAnchorOffsets(self:GetDimension(nLeft),self:GetDimension(nTop),self:GetDimension(nRight),self:GetDimension(nBottom))
end

function LUI_Frames:ScaleAll(wnd)
    self:Scale(wnd)

    if #wnd:GetChildren() > 0 then
        for _,child in pairs(wnd:GetChildren()) do
            self:ScaleAll(child)
        end
    end
end

-- #########################################################################################################################################
-- # SETTINGS
-- #########################################################################################################################################

function LUI_Frames:OnConfigure()
    self:OnToggleMenu()
end

function LUI_Frames:OnToggleMenu()
    if self.wndSettings then
        if self.wndSettings:IsShown() then
            self.wndSettings:Close()
        else
            self.wndSettings:Invoke()
        end
    end
end

function LUI_Frames:OnTabBtn(wndHandler, wndControl)
    for _,child in pairs(self.wndSettings:FindChild("Content"):GetChildren()) do
        if child:GetName() == wndControl:GetName() then
            child:Show(true)
        else
            child:Show(false)
        end
    end
end

function LUI_Frames:LoadSettings()
    self.wndSettings = Apollo.LoadForm(self.xmlDoc, "Settings", nil, self)
    self.wndSettings:Show(false,true)

    -- Tab
    for _,child in pairs(self.wndSettings:FindChild("NavContainer"):GetChildren()) do
        child:SetCheck(child:GetName() == "General")
    end

    for _,child in pairs(self.wndSettings:FindChild("Content"):GetChildren()) do
        child:Show(child:GetName() == "General")
    end

    -- Donation
    self.wndSettings:FindChild("DonateForm"):Show(false,true)
    self.wndSettings:FindChild("DonateSeperator"):Show(false,true)
    self.wndSettings:FindChild("DonateBtn"):Show((GameLib.GetRealmName() == "Jabbit"),true)
    self.wndSettings:FindChild("DonateForm"):FindChild("DonateSendBtn"):Enable(false)

    -- Spacing
    self.wndSettings:FindChild("Spacing"):FindChild("SliderText"):SetText(self.settings.spacing or 0)
    self.wndSettings:FindChild("Spacing"):FindChild("Slider"):SetValue(self.settings.spacing or 0)
    self.wndSettings:FindChild("Spacing"):FindChild("Slider"):SetData("spacing")

    -- Thickness
    self.wndSettings:FindChild("Thickness"):FindChild("SliderText"):SetText((self.settings.thickness or 0))
    self.wndSettings:FindChild("Thickness"):FindChild("Slider"):SetValue(self.settings.thickness or 0)
    self.wndSettings:FindChild("Thickness"):FindChild("Slider"):SetData("thickness")

    -- Cube Size
    self.wndSettings:FindChild("Cubes"):FindChild("SliderText"):SetText((self.settings.cubeSize or 0))
    self.wndSettings:FindChild("Cubes"):FindChild("Slider"):SetValue(self.settings.cubeSize or 0)
    self.wndSettings:FindChild("Cubes"):FindChild("Slider"):SetData("cubeSize")

    -- Frame Strata
    self.wndSettings:FindChild("General"):FindChild("FrameStrata"):FindChild("SliderText"):SetText((self.settings.strata or 0))
    self.wndSettings:FindChild("General"):FindChild("FrameStrata"):FindChild("Slider"):SetValue(self.settings.strata or 0)
    self.wndSettings:FindChild("General"):FindChild("FrameStrata"):FindChild("Slider"):SetData("strata")

    -- Shadow Checkbox
    self.wndSettings:FindChild("EnableShadow"):SetCheck(self.settings.showShadow or false)
    self.wndSettings:FindChild("EnableShadow"):SetData("showShadow")

    -- UI Scale Checkbox
    self.wndSettings:FindChild("ScaleCheckbox"):SetCheck(self.settings.useUIScale or false)
    self.wndSettings:FindChild("ScaleCheckbox"):SetData("useUIScale")

    -- Chat Panel
    self.wndSettings:FindChild("Chat"):FindChild("LengthHorizontal"):FindChild("SliderText"):SetText((self.settings.chat.length_horizontal or 0))
    self.wndSettings:FindChild("Chat"):FindChild("LengthHorizontal"):FindChild("Slider"):SetValue(self.settings.chat.length_horizontal or 0)
    self.wndSettings:FindChild("Chat"):FindChild("LengthHorizontal"):FindChild("Slider"):SetData({"chat","length_horizontal"})

    self.wndSettings:FindChild("Chat"):FindChild("LengthVertical"):FindChild("SliderText"):SetText((self.settings.chat.length_vertical or 0))
    self.wndSettings:FindChild("Chat"):FindChild("LengthVertical"):FindChild("Slider"):SetValue(self.settings.chat.length_vertical or 0)
    self.wndSettings:FindChild("Chat"):FindChild("LengthVertical"):FindChild("Slider"):SetData({"chat","length_vertical"})

    self.wndSettings:FindChild("Chat"):FindChild("FrameStrata"):FindChild("SliderText"):SetText((self.settings.chat.strata or 0))
    self.wndSettings:FindChild("Chat"):FindChild("FrameStrata"):FindChild("Slider"):SetValue(self.settings.chat.strata or 0)
    self.wndSettings:FindChild("Chat"):FindChild("FrameStrata"):FindChild("Slider"):SetData({"chat","strata"})

    self.wndSettings:FindChild("Chat"):FindChild("Position"):FindChild("XPosText"):SetText(self.settings.chat.posX)
    self.wndSettings:FindChild("Chat"):FindChild("Position"):FindChild("XPosText"):SetData({"chat","posX"})

    self.wndSettings:FindChild("Chat"):FindChild("Position"):FindChild("YPosText"):SetText(self.settings.chat.posY)
    self.wndSettings:FindChild("Chat"):FindChild("Position"):FindChild("YPosText"):SetData({"chat","posY"})

    self.wndSettings:FindChild("Chat"):FindChild("LinesCheckbox"):SetCheck(self.settings.chat.showLines or false)
    self.wndSettings:FindChild("Chat"):FindChild("LinesCheckbox"):SetData({"chat","showLines"})

    self.wndSettings:FindChild("Chat"):FindChild("StoreCheckbox"):SetCheck(self.settings.chat.showStore or false)
    self.wndSettings:FindChild("Chat"):FindChild("StoreCheckbox"):SetData({"chat","showStore"})

    self.wndSettings:FindChild("Chat"):FindChild("FayCheckbox"):SetCheck(self.settings.chat.showMadameFay or false)
    self.wndSettings:FindChild("Chat"):FindChild("FayCheckbox"):SetData({"chat","showMadameFay"})

    self.wndSettings:FindChild("Chat"):FindChild("EventCheckbox"):SetCheck(self.settings.chat.showEvent or false)
    self.wndSettings:FindChild("Chat"):FindChild("EventCheckbox"):SetData({"chat","showEvent"})

    -- Info Panel
    self.wndSettings:FindChild("Info"):FindChild("LengthHorizontal"):FindChild("SliderText"):SetText((self.settings.info.length_horizontal or 0))
    self.wndSettings:FindChild("Info"):FindChild("LengthHorizontal"):FindChild("Slider"):SetValue(self.settings.info.length_horizontal or 0)
    self.wndSettings:FindChild("Info"):FindChild("LengthHorizontal"):FindChild("Slider"):SetData({"info","length_horizontal"})

    self.wndSettings:FindChild("Info"):FindChild("LengthVertical"):FindChild("SliderText"):SetText((self.settings.info.length_vertical or 0))
    self.wndSettings:FindChild("Info"):FindChild("LengthVertical"):FindChild("Slider"):SetValue(self.settings.info.length_vertical or 0)
    self.wndSettings:FindChild("Info"):FindChild("LengthVertical"):FindChild("Slider"):SetData({"info","length_vertical"})

    self.wndSettings:FindChild("Info"):FindChild("FrameStrata"):FindChild("SliderText"):SetText((self.settings.info.strata or 0))
    self.wndSettings:FindChild("Info"):FindChild("FrameStrata"):FindChild("Slider"):SetValue(self.settings.info.strata or 0)
    self.wndSettings:FindChild("Info"):FindChild("FrameStrata"):FindChild("Slider"):SetData({"info","strata"})

    self.wndSettings:FindChild("Info"):FindChild("LinesCheckbox"):SetCheck(self.settings.info.showLines or false)
    self.wndSettings:FindChild("Info"):FindChild("LinesCheckbox"):SetData({"info","showLines"})

    self.wndSettings:FindChild("Info"):FindChild("XPCheckbox"):SetCheck(self.settings.info.showXP or false)
    self.wndSettings:FindChild("Info"):FindChild("XPCheckbox"):SetData({"info","showXP"})

    self.wndSettings:FindChild("Info"):FindChild("TimeCheckbox"):SetCheck(self.settings.info.showTime or false)
    self.wndSettings:FindChild("Info"):FindChild("TimeCheckbox"):SetData({"info","showTime"})

    self.wndSettings:FindChild("Info"):FindChild("TimeFormatCheckbox"):SetCheck(self.settings.info.b24HourFormat or false)
    self.wndSettings:FindChild("Info"):FindChild("TimeFormatCheckbox"):SetData({"info","b24HourFormat"})


    self.wndSettings:FindChild("Info"):FindChild("Position"):FindChild("XPosText"):SetText(self.settings.info.posX)
    self.wndSettings:FindChild("Info"):FindChild("Position"):FindChild("XPosText"):SetData({"info","posX"})

    self.wndSettings:FindChild("Info"):FindChild("Position"):FindChild("YPosText"):SetText(self.settings.info.posY)
    self.wndSettings:FindChild("Info"):FindChild("Position"):FindChild("YPosText"):SetData({"info","posY"})

    -- Minimap
    self.wndSettings:FindChild("Minimap"):FindChild("LengthHorizontal"):FindChild("SliderText"):SetText((self.settings.minimap.length_horizontal or 0))
    self.wndSettings:FindChild("Minimap"):FindChild("LengthHorizontal"):FindChild("Slider"):SetValue(self.settings.minimap.length_horizontal or 0)
    self.wndSettings:FindChild("Minimap"):FindChild("LengthHorizontal"):FindChild("Slider"):SetData({"minimap","length_horizontal"})

    self.wndSettings:FindChild("Minimap"):FindChild("LengthVertical"):FindChild("SliderText"):SetText((self.settings.minimap.length_vertical or 0))
    self.wndSettings:FindChild("Minimap"):FindChild("LengthVertical"):FindChild("Slider"):SetValue(self.settings.minimap.length_vertical or 0)
    self.wndSettings:FindChild("Minimap"):FindChild("LengthVertical"):FindChild("Slider"):SetData({"minimap","length_vertical"})

    self.wndSettings:FindChild("Minimap"):FindChild("Width"):FindChild("SliderText"):SetText((self.settings.minimap.width or 0))
    self.wndSettings:FindChild("Minimap"):FindChild("Width"):FindChild("Slider"):SetValue(self.settings.minimap.width or 0)
    self.wndSettings:FindChild("Minimap"):FindChild("Width"):FindChild("Slider"):SetData({"minimap","width"})

    self.wndSettings:FindChild("Minimap"):FindChild("Height"):FindChild("SliderText"):SetText((self.settings.minimap.height or 0))
    self.wndSettings:FindChild("Minimap"):FindChild("Height"):FindChild("Slider"):SetValue(self.settings.minimap.height or 0)
    self.wndSettings:FindChild("Minimap"):FindChild("Height"):FindChild("Slider"):SetData({"minimap","height"})

    self.wndSettings:FindChild("Minimap"):FindChild("Position"):FindChild("XPosText"):SetText(self.settings.minimap.posX)
    self.wndSettings:FindChild("Minimap"):FindChild("Position"):FindChild("XPosText"):SetData({"minimap","posX"})

    self.wndSettings:FindChild("Minimap"):FindChild("Position"):FindChild("YPosText"):SetText(self.settings.minimap.posY)
    self.wndSettings:FindChild("Minimap"):FindChild("Position"):FindChild("YPosText"):SetData({"minimap","posY"})

    self.wndSettings:FindChild("Minimap"):FindChild("EnableCheckbox"):SetCheck(self.settings.minimap.enable or false)
    self.wndSettings:FindChild("Minimap"):FindChild("EnableCheckbox"):SetData({"minimap","enable"})

    self.wndSettings:FindChild("Minimap"):FindChild("LinesCheckbox"):SetCheck(self.settings.minimap.showLines or false)
    self.wndSettings:FindChild("Minimap"):FindChild("LinesCheckbox"):SetData({"minimap","showLines"})

    self.wndSettings:FindChild("Minimap"):FindChild("ConnectorCheckbox"):SetCheck(self.settings.minimap.showConnectors or false)
    self.wndSettings:FindChild("Minimap"):FindChild("ConnectorCheckbox"):SetData({"minimap","showConnectors"})

    -- Sidebar
    self.wndSettings:FindChild("Sidebar"):FindChild("Width"):FindChild("SliderText"):SetText((self.settings.sidebar.width or 0))
    self.wndSettings:FindChild("Sidebar"):FindChild("Width"):FindChild("Slider"):SetValue(self.settings.sidebar.width or 0)
    self.wndSettings:FindChild("Sidebar"):FindChild("Width"):FindChild("Slider"):SetData({"sidebar","width"})

    self.wndSettings:FindChild("Sidebar"):FindChild("Height"):FindChild("SliderText"):SetText((self.settings.sidebar.height or 0))
    self.wndSettings:FindChild("Sidebar"):FindChild("Height"):FindChild("Slider"):SetValue(self.settings.sidebar.height or 0)
    self.wndSettings:FindChild("Sidebar"):FindChild("Height"):FindChild("Slider"):SetData({"sidebar","height"})

    self.wndSettings:FindChild("Sidebar"):FindChild("Position"):FindChild("XPosText"):SetText(self.settings.sidebar.posX)
    self.wndSettings:FindChild("Sidebar"):FindChild("Position"):FindChild("XPosText"):SetData({"sidebar","posX"})

    self.wndSettings:FindChild("Sidebar"):FindChild("Position"):FindChild("YPosText"):SetText(self.settings.sidebar.posY)
    self.wndSettings:FindChild("Sidebar"):FindChild("Position"):FindChild("YPosText"):SetData({"sidebar","posY"})

    self.wndSettings:FindChild("Sidebar"):FindChild("EnableCheckbox"):SetCheck(self.settings.sidebar.enable or false)
    self.wndSettings:FindChild("Sidebar"):FindChild("EnableCheckbox"):SetData({"sidebar","enable"})

    -- Actionbar
    self.wndSettings:FindChild("Actionbar"):FindChild("Width"):FindChild("SliderText"):SetText((self.settings.actionbar.width or 0))
    self.wndSettings:FindChild("Actionbar"):FindChild("Width"):FindChild("Slider"):SetValue(self.settings.actionbar.width or 0)
    self.wndSettings:FindChild("Actionbar"):FindChild("Width"):FindChild("Slider"):SetData({"actionbar","width"})

    self.wndSettings:FindChild("Actionbar"):FindChild("Height"):FindChild("SliderText"):SetText((self.settings.actionbar.height or 0))
    self.wndSettings:FindChild("Actionbar"):FindChild("Height"):FindChild("Slider"):SetValue(self.settings.actionbar.height or 0)
    self.wndSettings:FindChild("Actionbar"):FindChild("Height"):FindChild("Slider"):SetData({"actionbar","height"})

    self.wndSettings:FindChild("Actionbar"):FindChild("Position"):FindChild("XPosText"):SetText(self.settings.actionbar.posX)
    self.wndSettings:FindChild("Actionbar"):FindChild("Position"):FindChild("XPosText"):SetData({"actionbar","posX"})

    self.wndSettings:FindChild("Actionbar"):FindChild("Position"):FindChild("YPosText"):SetText(self.settings.actionbar.posY)
    self.wndSettings:FindChild("Actionbar"):FindChild("Position"):FindChild("YPosText"):SetData({"actionbar","posY"})

    self.wndSettings:FindChild("Actionbar"):FindChild("EnableCheckbox"):SetCheck(self.settings.actionbar.enable or false)
    self.wndSettings:FindChild("Actionbar"):FindChild("EnableCheckbox"):SetData({"actionbar","enable"})

    self.wndSettings:FindChild("Actionbar"):FindChild("LinesCheckbox"):SetCheck(self.settings.actionbar.showLines or false)
    self.wndSettings:FindChild("Actionbar"):FindChild("LinesCheckbox"):SetData({"actionbar","showLines"})

    self.wndSettings:FindChild("Actionbar"):FindChild("ElevateCheckbox"):SetCheck(self.settings.actionbar.elevate or false)
    self.wndSettings:FindChild("Actionbar"):FindChild("ElevateCheckbox"):SetData({"actionbar","elevate"})

    self.wndSettings:FindChild("Actionbar"):FindChild("Length1"):FindChild("SliderText"):SetText((self.settings.actionbar.line_length or 0))
    self.wndSettings:FindChild("Actionbar"):FindChild("Length1"):FindChild("Slider"):SetValue(self.settings.actionbar.line_length or 0)
    self.wndSettings:FindChild("Actionbar"):FindChild("Length1"):FindChild("Slider"):SetData({"actionbar","line_length"})

    self.wndSettings:FindChild("Actionbar"):FindChild("Length2"):FindChild("SliderText"):SetText((self.settings.actionbar.line_up or 0))
    self.wndSettings:FindChild("Actionbar"):FindChild("Length2"):FindChild("Slider"):SetValue(self.settings.actionbar.line_up or 0)
    self.wndSettings:FindChild("Actionbar"):FindChild("Length2"):FindChild("Slider"):SetData({"actionbar","line_up"})

    self.wndSettings:FindChild("Actionbar"):FindChild("Length3"):FindChild("SliderText"):SetText((self.settings.actionbar.line_side or 0))
    self.wndSettings:FindChild("Actionbar"):FindChild("Length3"):FindChild("Slider"):SetValue(self.settings.actionbar.line_side or 0)
    self.wndSettings:FindChild("Actionbar"):FindChild("Length3"):FindChild("Slider"):SetData({"actionbar","line_side"})

    self.wndSettings:FindChild("Actionbar"):FindChild("Length4"):FindChild("SliderText"):SetText((self.settings.actionbar.line_margin or 0))
    self.wndSettings:FindChild("Actionbar"):FindChild("Length4"):FindChild("Slider"):SetValue(self.settings.actionbar.line_margin or 0)
    self.wndSettings:FindChild("Actionbar"):FindChild("Length4"):FindChild("Slider"):SetData({"actionbar","line_margin"})

    self:ToggleSetting(self.settings.actionbar.elevate)
end

function LUI_Frames:ToggleSetting(state)
    local opacity = state and 1 or 0.5

    self.wndSettings:FindChild("Actionbar"):FindChild("Length2"):FindChild("Label"):SetOpacity(opacity,0)
    self.wndSettings:FindChild("Actionbar"):FindChild("Length2"):FindChild("SliderText"):Enable(state)
    self.wndSettings:FindChild("Actionbar"):FindChild("Length2"):FindChild("SliderText"):SetOpacity(opacity,0)
    self.wndSettings:FindChild("Actionbar"):FindChild("Length2"):FindChild("Slider"):Enable(state)
    self.wndSettings:FindChild("Actionbar"):FindChild("Length2"):FindChild("Slider"):SetOpacity(opacity,0)

    self.wndSettings:FindChild("Actionbar"):FindChild("Length3"):FindChild("Label"):SetOpacity(opacity,0)
    self.wndSettings:FindChild("Actionbar"):FindChild("Length3"):FindChild("SliderText"):Enable(state)
    self.wndSettings:FindChild("Actionbar"):FindChild("Length3"):FindChild("SliderText"):SetOpacity(opacity,0)
    self.wndSettings:FindChild("Actionbar"):FindChild("Length3"):FindChild("Slider"):Enable(state)
    self.wndSettings:FindChild("Actionbar"):FindChild("Length3"):FindChild("Slider"):SetOpacity(opacity,0)

    self.wndSettings:FindChild("Actionbar"):FindChild("Length4"):FindChild("Label"):SetOpacity(opacity,0)
    self.wndSettings:FindChild("Actionbar"):FindChild("Length4"):FindChild("SliderText"):Enable(state)
    self.wndSettings:FindChild("Actionbar"):FindChild("Length4"):FindChild("SliderText"):SetOpacity(opacity,0)
    self.wndSettings:FindChild("Actionbar"):FindChild("Length4"):FindChild("Slider"):Enable(state)
    self.wndSettings:FindChild("Actionbar"):FindChild("Length4"):FindChild("Slider"):SetOpacity(opacity,0)
end

function LUI_Frames:OnCheckbox(wndHandler, wndControl)
    local setting = wndControl:GetData()
    local value = wndHandler:IsChecked()

    if type(setting) == "string" then
        self.settings[setting] = value
    else
        self.settings[setting[1]][setting[2]] = value

        if setting[2] == "elevate" then
            self:ToggleSetting(value)
        end
    end

    self:Build()
end

function LUI_Frames:OnSliderChange(wndHandler, wndControl)
    local setting = wndControl:GetData()
    local value = wndHandler:GetValue()

    wndControl:GetParent():FindChild("SliderText"):SetText(value)

    if type(setting) == "string" then
        self.settings[setting] = value
    else
        self.settings[setting[1]][setting[2]] = value
    end

    self:Build()
end

function LUI_Frames:OnTextChange(wndHandler, wndControl)
    local setting = wndControl:GetData()
    local value = wndHandler:GetText()

    if not tonumber(value) then
        return
    end

    if type(setting) == "string" then
        self.settings[setting] = value
    else
        self.settings[setting[1]][setting[2]] = value
    end

    self:Build()
end

function LUI_Frames:Round(num, idp)
    local mult = 10^(idp or 0)
    return math.floor(num * mult + 0.5) / mult
end

function LUI_Frames:OnDonate(wndHandler, wndControl)
    self.wndSettings:FindChild("DonateSeperator"):Show(wndHandler:IsChecked(),true)
    self.wndSettings:FindChild("DonateForm"):Show(wndHandler:IsChecked(),true)
end

function LUI_Frames:OnDonationChanged(wndHandler, wndControl)
    local amount = self.wndSettings:FindChild("DonateForm"):FindChild("CashWindow"):GetAmount()
    local recipient = "Loui NaN"

    if self.unitPlayer:GetFaction() ~= 166 then
        recipient = "Loui x"
    end

    if amount >= 1 then
        self.wndSettings:FindChild("DonateForm"):FindChild("DonateSendBtn"):SetActionData(GameLib.CodeEnumConfirmButtonType.SendMail, recipient, "Jabbit", "LUI AugmentorHelper Donation", tostring(self.unitPlayer:GetName()) .. " donated something for you!", nil, MailSystemLib.MailDeliverySpeed_Instant, 0, self.wndSettings:FindChild("DonateForm"):FindChild("CashWindow"):GetCurrency())
        self.wndSettings:FindChild("DonateForm"):FindChild("DonateSendBtn"):Enable(true)
    else
        self.wndSettings:FindChild("DonateForm"):FindChild("DonateSendBtn"):Enable(false)
    end
end

function LUI_Frames:OnDonationSent()
    self.wndSettings:FindChild("DonateBtn"):SetCheck(false)
    self.wndSettings:FindChild("DonateForm"):Show(false)
    self.wndSettings:FindChild("DonateSeperator"):Show(false)
    self.wndSettings:FindChild("DonateForm"):FindChild("CashWindow"):SetAmount(0)
    self.wndSettings:FindChild("DonateForm"):FindChild("DonateSendBtn"):Enable(false)
    Print("Thank you very much!!! <3")
end

local LUI_FramesInst = LUI_Frames:new()
LUI_FramesInst:Init()
